"""Cost used for test data."""

SCHOOL_ID = "5C9756CEEA1B438A80C65C20D2720E23"
ACCOUNT_ID = "55503A65D2F448AC8DB5F8181B305281"
STUDENT_ONE_ID = "DC90F9C55C6142519C9CC9EFA1847922"
STUDENT_TWO_ID = "5A2AECC78C2E43FD9CC03CCE55D9806D"
PASSWORD = "NOTREAL"  # noqa: S105
USER_NAME = "TestUser"
SCHOOL_CODE = "88552"
